-----------------------------------------------------------------------------------------------
-- Client Lua Script for Nicknames
-- (C) Athica
-----------------------------------------------------------------------------------------------

require "Window" 
require "Apollo"
 
-----------------------------------------------------------------------------------------------
-- Nicknames Module Definition
-----------------------------------------------------------------------------------------------
local Nicknames = {} 
Nicknames.Version = "v0.1.3"

 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Nicknames:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    return o
end

function Nicknames:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
        "Gemini:Hook-1.0"
	}
	-- does not list ChatLog as dependency because of other ChatLog addons
	-- see function "GetHookAddon"
	
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)

end
 

-----------------------------------------------------------------------------------------------
-- Nicknames OnLoad
-----------------------------------------------------------------------------------------------
function Nicknames:OnLoad()
	
	self.settings = {}
	self.settings.nicknames = {}
	
	-- Hook into the method that builds chat-messages for printing
	Apollo.GetPackage("Gemini:Hook-1.0").tPackage:Embed(self)
	
	crb = self.GetHookAddon()
	if crb ~= nil then
	
	    self:RawHook(crb, "OnChatInputReturn")
		
	end	
	
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("Nicknames.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)

end

function Nicknames:OnDocLoaded()
	if self.xmlDoc == nil or not self.xmlDoc:IsLoaded() then return end

	-- Register handlers for events, slash commands and timer, etc.
	
	Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
	Apollo.RegisterEventHandler("Nicknames_Show", "SlashHandler", self)
end

function Nicknames:OnWindowManagementReady()
	Apollo.RegisterSlashCommand("nicknames",        "SlashHandler", self)
	Apollo.RegisterSlashCommand("nn",               "SlashHandler", self)
	
	self:InitUI()
end

function Nicknames:SlashHandler(cmd, arg)
	
	if not arg or arg == "" or arg == "show" then ----------------SHOW
	
		self:ShowUI()
	
	elseif arg == "list" then ------------------------------------LIST
	
		Print( 'Nicknames: listing nicknames... ' )
		for k, v in pairsByKeys( self.settings.nicknames ) do
			Print( ''..k..': '..v )
		end
		Print( 'Nicknames: -------------------- ' )
		
	elseif arg:sub(1,4) == "set " then ----------------------------SET
	
		local tokens = self:Split( arg )
		
		if not tokens[2] or not tokens[3] or not tokens[4] then
			Print( 'Nicknames: Incorrect syntax, use "set <nick> <real name>" ' )
			return
		end
		
		nick = tokens[2]:lower()
		target = tokens[3].." "..tokens[4]
		
		self.settings.nicknames[ nick ] = target
		
		Print( 'Nicknames: Set "'..nick..'" to point to "'..target..'"' )
		
		if self.wndMain:IsVisible() then self:RedrawAll() end
		
	elseif arg:sub(1, 4) == "del " then ----------------------------DEL
	
		local tokens = self:Split( arg )
		
		if not tokens[2] then
			Print( 'Nicknames: Incorrect syntax, use "del <nick>" ')
			return
		end
		
		if not self.settings.nicknames[ tokens[2] ] then
			Print( 'Nicknames: nickname "'..tokens[2]..'" does not exist ')
			return
		end
		
		self.settings.nicknames[ tokens[2] ] = nil
		
		Print( 'Nicknames: Removed "'..nick..'"' )

		if self.wndMain:IsVisible() then self:RedrawAll() end
		
	else
	
		Print( 'Nicknames: Try "show | set <nick> <real name>" | "del <nick>" | "list" ' )
		
	end
	
end

function Nicknames:OnChatInputReturn(luaCaller, wndHandler, wndControl, strText)

	if strText:lower():sub(1,3) == "/w " or      -- English
		strText:lower():sub(1,3) == "/t " or 
		strText:lower():sub(1,6) == "/tell " or
		strText:lower():sub(1,9) == "/whisper " or
		strText:lower():sub(1,6) == "/join " or
		strText:lower():sub(1,8) == "/invite " or
		
		strText:lower():sub(1,3)  == "/f " or    -- German
		strText:lower():sub(1,10) == "/flstern " or
		strText:lower():sub(1,10) == "/einladen " or
		strText:lower():sub(1,11) == "/beitreten " then
	
		strText = self:CheckReplaceNickname( strText )

	end

	-- Call the original function	
	crb = self:GetHookAddon()
	if crb ~= nil then
	
		self.hooks[crb].OnChatInputReturn(luaCaller, wndHandler, wndControl, strText)
		
	end

end

function Nicknames:CheckReplaceNickname( strText )
	local tokens = self:Split( strText )
	
	local lookupnick = tokens[2]:lower()
	if not lookupnick then return strText end
	
	local actualname = self.settings.nicknames[ lookupnick ]
	if not actualname then return strText end
	
	local nametokens = self:Split( actualname )
	if tokens[2] and tokens[3] and nametokens[2] and
	   tokens[2]:lower() == nametokens[1]:lower() and
	   tokens[3]:lower() == nametokens[2]:lower() then

		-- Skip
		-- FirstName and LastName are already in there*
		
	else

--		tokens[2] = lookupnick
--		strText = tokens[1].." "..tokens[2].." "..self:Join( tokens, " ", 3 )   -- Join ruins item links

--		local pos = strText:find( tokens[2] ) + tokens[2]:len() 
--		strText = tokens[1].." "..actualname..strText:sub(pos)     -- Still breaks item links
	
		strText = strText:gsub( tokens[2], actualname, 1 )     -- Still breaks item links

	end
	
	return strText
	
	--[[
	* if LastName is not already in there
	
	Imagine this situation:
		- nickname defined:  "athica" -> "Athica Amnell"
		- you get a whisper from "Athica Amnell"
		- if you now use /r or Ctrl+R (to reply), the game will change that to "/w Athica Amnell"
		- when you send the reply, the addon will replace this with "/w Athica Amnell Amnell"
		  because "athica" is a nickname
	
	--]]
end

-----------------------------------------------------------------------------------------------
-- Save/Restore
-----------------------------------------------------------------------------------------------
function Nicknames:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Realm then return nil end

	self.settings.version = self.Version
	
	if self.didFirstDraw then
		-- Store new window-size 
		self.settings.window = {}
		self.settings.window.left,
		self.settings.window.top,
		self.settings.window.right,
		self.settings.window.bottom = self.wndMain:GetAnchorOffsets()
	end
	
	return self.settings
end

function Nicknames:OnRestore(eLevel, tData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Realm then return nil end
	if not tData then return nil end
	
	if tData.nicknames then
		self.settings.nicknames = self:CopyTable( tData.nicknames )
	end
	
	if tData.window then
		self.settings.window = self:CopyTable( tData.window )
	end
end

-----------------------------------------------------------------------------------------------
-- Nicknames Functions
-----------------------------------------------------------------------------------------------

function Nicknames:GetHookAddon()
	-- Separate method to look for addon to hook, because re-use.

	crb = Apollo.GetAddon("ChatLog")
	--[[
	if crb == nil then
		crb = Apollo.GetAddon("BetterChatLog")
	end	
	if crb == nil then
		crb = Apollo.GetAddon("ImprovedChatLog")
	end	
	if crb == nil then
		crb = Apollo.GetAddon("ChatAdvanced")
	end	
	--]]		

	return crb
end

---------------------------------------------------------------------------------------
-- Helper functions
-----------------------------------------------------------------------------------------------

-- Helper function to copy a table
function Nicknames:CopyTable(t)
	local new = {}

	for k, v in pairs( t ) do
		if type( v ) == 'table' then
			new[k] = self:CopyTable(v)
		else
			new[k] = v
		end
	end

	return new
end

function Nicknames:Split(inputstr, sep)
	if sep == nil then
    	sep = "%s"
	end
	local t={} ; i=1
	for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
		t[i] = str
		i = i + 1
	end
	return t
end

function Nicknames:Join(inputtable, sep, start)
	if sep == nil then sep = " " end
	if start == nil then start = 1 end
	local s = ""
	local first = true
	for k, v in pairs(inputtable) do
		if k >= start then
	  		if not first then
				s = s .. sep
			end
			s = s .. v
			first = false
		end
	end
	return s
end

function pairsByKeys (t, f)
	local a = {}
	for n in pairs(t) do table.insert(a, n) end
	table.sort(a, f)
	local i = 0      -- iterator variable
	local iter = function ()   -- iterator function
		i = i + 1
		if a[i] == nil then return nil
		else return a[i], t[a[i]]
		end
	end
	return iter
end

-----------------------------------------------------------------------------------------------
-- Nicknames Instance
-----------------------------------------------------------------------------------------------
local NicknamesInst = Nicknames:new()
NicknamesInst:Init()
