-- UI

require "Window"

local Nicknames = Apollo.GetAddon("Nicknames")
local Info = Apollo.GetAddonInfo("Nicknames")

function Nicknames:InitUI()

    self.wndMain = Apollo.LoadForm(self.xmlDoc, "NicknamesForm", nil, self)
	if self.wndMain == nil then
		Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
		return
	end

    self.wndMain:Show(false, true)

	self.wndMain:SetSizingMinimum( 300, 300 )
	self.wndMain:SetSizingMaximum( 1000, 2000 )
	
end

function Nicknames:ShowUI()

	if not self.didFirstDraw then
		Print("hoi1")
		self.didFirstDraw = true
		
		-- Restore window settings
		if self.settings.window then
			Print("hoi2")
			self.wndMain:SetAnchorOffsets(
			self.settings.window.left,
			self.settings.window.top,
			self.settings.window.right,
			self.settings.window.bottom)
		end
	end

	self.wndMain:FindChild( "wndNew" ):Show( false, true )

	self:RedrawAll()
	
	self.wndMain:Invoke() -- show the window

end

function Nicknames:RedrawAll()

	local wndParent = self.wndMain:FindChild("wndScrollList")
	if not wndParent then return end
	wndParent:DestroyChildren()
	
	for nick, actual in pairsByKeys( self.settings.nicknames ) do
		
		-- create new header
		self:NewNicknameEntry( wndParent, nick, actual )
	
	end
	
	wndParent:ArrangeChildrenVert(0)
	wndParent:RecalculateContentExtents()
	
end

function Nicknames:NewNicknameEntry( wndParent, nick, actual )

	local wndEntry = Apollo.LoadForm(self.xmlDoc, "NicknameEntry", wndParent, self)
	wndEntry:SetData( nick )
	
	local wndNickname = wndEntry:FindChild("wndNickname")
	local wndActual = wndEntry:FindChild("wndActualName")

	wndNickname:SetText( nick )
	wndActual:SetText( actual )	

end

function Nicknames:OnInterfaceMenuListHasLoaded()
	
	-- Communicate with the 'button in the bottom left corner'
	Event_FireGenericEvent(
			"InterfaceMenuList_NewAddOn",
			"Nicknames",
				{"Nicknames_Show",
				"",
				""})
 
	self:UpdateInterfaceMenuAlerts()
end

function Nicknames:UpdateInterfaceMenuAlerts()
	Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", Nicknames, {false, "Nicknames", 0})
end

---------------------------------------------------------------------------------------------------
-- NicknamesForm Functions
---------------------------------------------------------------------------------------------------

function Nicknames:OnMainWindowClose( wndHandler, wndControl, eMouseButton )

	self.wndMain:Close()
	
end

function Nicknames:OnBtnNewClick( wndHandler, wndControl, eMouseButton )
	local wndNew = self.wndMain:FindChild( "wndNew" )
	wndNew:Show( not wndNew:IsVisible() )
end

function Nicknames:OnWndNewShow( wndHandler, wndControl )
	local ebNick  = wndControl:FindChild( "ebNickname" )
	local ebFirst = wndControl:FindChild( "ebFirstName" )
	local ebLast  = wndControl:FindChild( "ebLastName" )
	
	ebNick:SetText( "" )
	ebFirst:SetText( "" )
	ebLast:SetText( "" )
	
	ebNick:SetFocus()
end

function Nicknames:OnBtnCloseNewClick( wndHandler, wndControl, eMouseButton )
	self.wndMain:FindChild( "wndNew" ):Show( false )
end

function trim12( s )
	local from = s:match"^%s*()"
	return from > #s and "" or s:match(".*%S", from)
end

function Nicknames:OnBtnCreateNewClick( wndHandler, wndControl, eMouseButton )
	local nick  = wndControl:GetParent():FindChild( "ebNickname"):GetText()
	local first = wndControl:GetParent():FindChild( "ebFirstName"):GetText()
	local last  = wndControl:GetParent():FindChild( "ebLastName"):GetText()
	
	if not nick
	or not first
	or not last
	or trim12(nick) == ""
	or trim12(first) == ""
	or trim12(last) == "" then return end
	
	self.settings.nicknames[ nick:lower() ] = first.." "..last

	wndControl:GetParent():Show( false )
	
	self:RedrawAll()
end

function Nicknames:OnLastNameEditBoxReturn( wndHandler, wndControl, strText )
	self:OnBtnCreateNewClick( wndHandler, wndControl, 0 )
end

---------------------------------------------------------------------------------------------------
-- NicknameEntry Functions
---------------------------------------------------------------------------------------------------

function Nicknames:OnDeleteNicknameEntryClick( wndHandler, wndControl, eMouseButton )

	local nick = wndControl:GetParent():GetParent():GetData()
	if not nick then return end

	self.settings.nicknames[ nick ] = nil
	
	self:RedrawAll()	

end
